/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef CONEXUSIO_H
#define CONEXUSIO_H

#include <set>
#include <queue>

#include <sys/select.h>

#include <glibmm/dispatcher.h>
#include <glibmm/thread.h>

#include <conexus/pointer.h>
#include <conexus/enums.h>
#include <conexus/except.h>
#include <conexus/data.h>

/**
 * \mainpage Conexus Library Programmer's Reference Guide
 *
 * This is the programmer's reference guide for the Conexus library.
 *
 * \section manual_io Conexus I/O Concepts
 * This page introduces the general class structure of the Conexus library.
 * For the most part, the following links just contain examples, but a few have
 * more in-depth discussions on usage.
 *
 * <h2>Sections:</h2>
 * - \subpage general
 *   - \ref signals_and_slots
 *   - \ref smart_pointers
 * - \subpage io_endpoints
 *   - \subpage receiving_data_via_callback
 *     - \subpage datagram_example
 *   - \subpage reading_data
 *   - \subpage receiving_notifications
 *   - \subpage blocking_write
 *   - \subpage write_notification
 *   - \subpage nonblocking_write
 * - \subpage Inheritance
 *   - \subpage manual_server
 *   - \subpage manual_endpoint
 *     - \subpage manual_filedescriptor
 *       - \subpage manual_socket
 *         - \subpage manual_ipv4
 *           - \subpage manual_ipv4_udp
 *           - \subpage manual_ipv4_udpposet
 *           - \subpage manual_ipv4_tcp
 *           - \subpage manual_ipv4_tcp_server
 *         - \subpage manual_ipv6
 *           - \subpage manual_ipv6_udp
 *           - \subpage manual_ipv6_udpposet
 *           - \subpage manual_ipv6_tcp
 *           - \subpage manual_ipv6_tcp_server
 *         - \subpage manual_packet
 *       - \subpage manual_serial
 *         - \subpage manual_tty
 *
 * \page general General Overview
 * The purpose of the Conexus library is to provide a generalized I/O framework that supports asynchronous
 * multi-threaded communications via the signal/slot mechanism of the sigc++ library, allowing for seamless
 * integration with Gtkmm applications.
 *
 * \section signals_and_slots sigc++ Signals & Slots
 * The sigc++ library defines slots as "a container for an arbitrary functor" and signals as "a list of slots
 * that are executed on emission." In other words, sigc++ provides a signal/callback system for defining callbacks
 * to be connected to a particular signal. When the signal is emitted, the callback (or slot) is executed.
 *
 * \section smart_pointers Smart Pointers
 * The Conexus library relies upon smart pointers for several aspects, with the most prevalent being delivery
 * of data from the I/O mechnisms and embodied in the Conexus::Data::pointer typedef.
 *
 * \page io_endpoints I/O Endpoints
 * The Conexus library is centered around the concept of generic I/O endpoints. The Conexus::Endpoint class
 * embodies the concepts of a generic I/O communicator, and conceptually is very close to a socket. However,
 * the inheritance hierarchy imposed allows classes that derive from Conexus::Endpoint to leverage polymorphic behavior.
 *
 * An endpoint serves to abstract the details of communication. Various APIs such as the BSD Socket API, POSIX File
 * I/O API, pcap (Packet Capture) library API each have their own syntax and mechanisms for communication. The following
 * shows several communication mechanisms and a sampling of the knowledge (functions, ioctls, etc.) associated with
 * each mechanism.
 *
 * \image html problem.png "Example communication mechanisms and sampling of functions, ioctls etc. associated with each mechanism"
 *
 * An endpoint abstracts these details to a minimum set of communication mechanisms that are similar to the POSIX API.
 *
 * \image html endpoints.png "An endpoint abstracts many, but not all, I/O concepts. Children of Conexus::Endpoint are responsible for mechanism specifics."
 *
 * Since each of the communication mechanisms have unique aspects, such as the TCP and UDP protocols concept of a port,
 * children of Conexus::Endpoint such as Conexus::IPv4::TCP, Conexus::IPv4::UDP and Conexus::TTY provide the additional functionality
 * for each mechanism.
 *
 * An endpoint supports the following generic communication mechanisms:
 *  -# Delivery of data from an external source via callbacks
 *  -# Notification when data has been transmitted from the source
 *  -# Reading of data from the endpoint
 *
 * \page receiving_data_via_callback Receiving data from endpoints via a callback
 * An endpoint can deliver data as it is received to one or more functions or class methods within your application.
 * Conceptually, this is the delivery mechanism that is probably most appropriate for datagram delivery such as UDP/IP.
 *
 * To receive data from an endpoint via callback you need to:
 *  -# Create the function or class method that will receive the data from the endpoint.
 *    - Your function or method must have the following signature:
 *      -# \c void \e function_or_method_name \c (const \c Conexus::Data::pointer)
 *      - This means that the return type of the function or method must be \c void and the first parameter must be
 *        \c const \c Conexus::Data::pointer
 *      - If you want other parameters you must use \c sigc::bind , which is discussed in the Gtkmm tutorial and the sigc++ documentation.
 *  -# Create the sigc++ slot that will receive the data from the endpoint.
 *    - Generally, \c sigc::mem_fun and \c sigc::ptr_fun will suffice
 *    - The required signature of the sigc::slot is:
 *      -# \c sigc::slot<void, \c Conexus::Data::pointer>
 *  -# Connect the slot to the endpoint using \c signal_data()
 *
 * The following image illustrates the sequence of operations in receiving data via callbacks.
 * \image html callback.png Sequence of operations required to receive data via a callback mechanism.
 *
 * \page datagram_example Examples: Connecting an endpoint to a function and to a class method
 *
 * \section function_example Example: A function callback
 * The following example illustrates the elements necessary to connect a Conexus::Endpoint object to a callback function.
 *
 * \dontinclude udp_function.cpp
 * \skip include
 * \until }
 * \until }
 * \until }
 *
 * \section method_example Example: A class method callback
 * The following example illustrates the elements necessary to connect a Conexus::Endpoint object to a callback class method.
 *
 * \dontinclude udp_method.cpp
 * \skip include
 * \until }
 * \until }
 * \until }
 * \until }
 *
 * Notice that the main difference between the two examples is the use of the sigc++ library mechanisms for creating function
 * slots using \c sigc::ptr_fun and class member slots using \c sigc::mem_fun .
 *
 * \page reading_data Reading Data
 *
 * The Conexus::Endpoint class provides a pure virtual method with the following prototype for reading data from an endpoint:
 * \code
 * virtual Data read(size_t max_octets = 0) throw (read_exception) = 0;
 * \endcode
 * This method is similar to the POSIX API read method, which has the following prototype:
 * \code
 * ssize_t read(int fd, void *buf, size_t count);
 * \endcode
 * The following list examines the differences between the POSIX API \c read() function and the Conexus library \c read() method,
 * and explains the reasons for differing.
 *  -# \b Difference: The return value is an instance of Conexus::Data::pointer rather than the number of bytes read.
 *    - \b Reason: The return value is a pointer to the data read from the endpoint \e and the size in bytes of that data embodied
 *         in the Conexus::Data class. This is due to the fact that other objects may be connected to the endpoint, and a \c read()
 *         operation will not only return the data as a result of calling the method, but will also emit any signals necessary
 *         including delivery of data to those connected to the endpoint along with sending notifications of a read operation.
 *  -# \b Difference: The Conexus \c read() method does not accept a file descriptor as a parameter.
 *    - \b Reason: The Conexus::Endpoint class is intended to provide an interface at a higher level of abstraction than POSIX file I/O.
 *         Child classes such as Conexus::FileDescriptor encapsulate the concept of POSIX file descriptors. You will note that this
 *         class already has knowledge of the associated file descriptor, and thus the file descriptor parameter is not necessary
 *         even at lower levels of implementation.
 *  -# \b Difference: The Conexus \c read() method does not accept a pointer in which the read data will be stored.
 *    - \b Reason: The return value contains a pointer to the read data. This is necessary since a read operation may send the data read
 *         not only as a return value, but also provide copies to all objects connected to the endpoint.
 *
 * Despite the differences, there remains a similarity between the \c max_octets and the \c count parameter. In the POSIX API the \c count
 * parameter is used to indicate that \c buf points to an allocated storage area of at least \c count octets, and additionally places
 * an upper limit on the number of octets that will be read in. The \c max_octets fills the latter role, and indicates the upper limit
 * on the octets to read in. As with the POSIX API, the actual number of octets read may not be equal to the requested octets. The
 * return Conexus::Data object's \c size member will indicate the actual number of octets read, while the \c data member will contain
 * a pointer to the actual data.
 *
 * The following image illustrates the sequence of operations in receiving data via the \c read() method.
 * \image html read.png Sequence of operations that occur when the \c read() method is called.
 *
 * When a callback has been registered with the endpoint a \c read() operation will simultaneously deliver the endpoint data to
 * the \c read() method caller and to all connected callbacks.
 *
 * The following image illustrates the sequence of operations in receiving data via the \c read() method when a callback has been
 * registered with the endpoint.
 * \image html read_with_callback.png Sequence of operations that occur when the \c read() method is called and a callback has been registered with the endpoint.
 *
 * \page receiving_notifications Receiving Notifications
 *
 * \image html data_received_notification.png Sequence of operations necessary to receive notifications from endpoint when data has been received from external source.
 * \image html read_on_notification.png Using read notifications to gain greater control over data flow from an endpoint.
 *
 * \page blocking_write Blocking Writes
 * \image html blocking_write.png Sequence of operations that occur when the \c write() method is called with a default blocking parameter.
 *
 * \page write_notification Write Notification
 * \image html write_notification.png Sequence of operations that are necessary to receive notifications when an endpoint performs a write operation.
 *
 * \page nonblocking_write Nonblocking Writes
 * \image html nonblocking_write.png Using nonblocking \c write() calls combined with write notifications.
 *
 * \page Inheritance The inheritance hierarchy
 * The Conexus library inheritance is rooted at the Conexus::Server class and descends
 * into less abstract classes with leaves such as the Conexus::IP::UDP class.
 *
 * \page manual_server Servers
 * The root of the Conexus inheritance hierarchy is the Conexus::Server class. This class inherits
 * from sigc::trackable and provides the concept of a data source that may be started, stopped and
 * connected to via sigc++'s signal/slot mechanism.
 *
 * \page manual_endpoint Conexus::Endpoint
 *
 *
 */

namespace Conexus
  {

  /**
   * This class is the top level object for all subclasses performing I/O (Input/Output)
   * operations.
   *
   * read/write methods are pure virtual methods in this base class and it is the responsibility
   * of all children to provide proper implementations that may go beyond the traditional read(2)
   * and write(2) functions standard in Linux/Unix.
   *
   * Children are also responsible for maintenance of the file descriptor member @em m_fd and the
   * integer state value @em m_state.
   *
   * This class inherits from propertymm::Object and thus makes some properties available via
   * the propertymm framework as well as providing direct accessor methods.
   *
   * @author Rick L Vinyard Jr
   * @ingroup conexus
   */
  class Endpoint
    {
      public:

      /** Class scope smart pointer typedef */
      typedef ConexusPointer<Endpoint> pointer;

      /**
       * Default constructor setting state to 0 and file descriptor to -1 (which should be an
       * invalid fd on just about every POSIX system.
       *
       * This constructor simply sets the values according to the parameters and performs no
       * real actions with the file descriptor.
       */
      Endpoint(bool close_on_destruct=true);

      /**
       * Destructor does nothing. It will not call @em close() upon the file descriptor or perform
       * any other cleanup; these are the responsibility of the children.
       */
      virtual ~Endpoint();

      /**
       * Start the endpoint. If the endpoint is configured for multi-threaded
       * operation this will create and start the read thread.
       *
       * Whether a dispatcher is used or not is dependent upon the global setting
       * which defaults to false, but may be set with Conexus::init() or
       * Conexus::set_use_dispatcher_default().
       *
       * If you want to handle it manually, endpoints are capable of reading/writing
       * without ever calling start/stop.
       */
      virtual void start( );

      /**
       * Starts the endpoint, and forces it to use a specific dispatcher value possibly overriding
       * the global default.
       */
      virtual void start( bool use_dispatcher );

      /**
       * Stops any running threads. Stopping is not immediate. If a read operation is being performed
       * the current read will complete before the thread is stopped.
       */
      virtual void stop( );

      /** True if the endpoint is running with threaded reads */
      virtual bool is_running();

      /** True if received data is timestamped */
      virtual bool timestamp_received_data();

      /** Set to true to timestamp received data */
      virtual void set_timestamp_received_data(bool t=true);

      virtual void set_blocking_write(bool block=true);

      virtual bool is_blocking_write();

      /**
       * Signal emitted when data is read. Connect to this signal if you want to receive all
       * data read by the endpoint. The data object received is unique for each callback
       * point, allowing a callback to modify the data set without effect subsequent callbacks.
       */
      sigc::signal<void, const Data> signal_data();

      /**
       * Signal emitted when data is received notifying the callback of the number of bytes read.
       * Connect to this signal if you need to know when data is received, and how much, but don't
       * want the data contents.
       */
      sigc::signal<void, size_t> signal_data_received();

      /**
       * Signal emitted when data is transmitted, notifying the callback of the number of bytes transmitted.
       * Connect to this signal if you need to know when data is transmitted, and how much, but don't
       * want the transmitted data contents.
       */
      sigc::signal<void, size_t> signal_data_transmitted();

      long int read_timeout();

      void set_read_timeout( long int t );

      long int write_timeout();

      void set_write_timeout( long int t );

      bool read_terminate_immediate();

      void set_read_terminate_immediate( bool i=true );

      bool write_terminate_immediate();

      void set_write_terminate_immediate( bool i=true );

      /**
       * Will attempt to open the object without any additional information.
       */
      virtual void open() throw (open_exception) = 0;

      /**
       */
      virtual void close(bool force=false) throw (close_exception) = 0;

      /**
       * A pure virtual method that must be reimplemented by children to perform whatever actions
       * necessary to ensure writing/transmission of a block of data pointed to by @em data and of
       * @em size bytes.
       *
       * @return The number of bytes actually written.
       * @param data Pointer to the raw data block to be written.
       * @param size Size in bytes of the raw data block to write.
       */
      ssize_t write(const void* data, size_t size) throw (write_exception);

      /**
       * Write data according to virtual method defined by children.
       * @return The number of bytes actually written.
       */
      ssize_t write(const Data data) throw (write_exception);

      /**
       * A pure virtual method that must be reimplemented by children to perform whatever actions
       * necessary to read a block of data. Children may utilize the parameter @em s or may choose
       * to ignore the parameter entirely.
       *
       * @return A smart pointer to a Data object containing the data and size of data in bytes. The data
       *    is dynamically allocated via a smart pointer, and deallocated in accordance
       *    with the rules for smart pointers. Therefore, if you want to keep the data
       *    from being automatically deallocated, keep the returned data object (or a
       *    copy of the returned data object) around as long as you need it. Call by value
       *    is sufficient to keep the object around, but extracting the pointer to the actual
       *    data via the dereference operator or through the smart pointer's get() method is
       *    not sufficient to keep the data block from being deallocated (again, keeping a
       *    copy around is easier).
       *
       * @param s The size of the data block to read. Suggested semantics for children are:
       *   - s=0: read as much as possible
       *   - s>0: read up to s bytes
       */
      Data read(size_t s = 0) throw (read_exception);

      virtual ssize_t input_available();

      /**
       * It is intended that children will provide their own implementation of
       * this method to take into account the actions necessary for any additional
       * states they may introduce.
       *
       * This method as implemented recognizes the OPEN, CLOSE and UNCHANGED states
       * and calls the virtual open, close (or no call) as is appropriate.
       *
       * This method does not actually modify the m_state member. It is the
       * responsibility of the called virtual methods open() and close() to
       * actually perform the modification.
       *
       * @return true if all requested state changes occurred, false otherwise
       */
      virtual void change_state(long new_state) throw (state_exception);

      /**
       * Return this endpoint's current numeric state.
       * The meaning of this value is specific to the endpoint.
       */
      long state();

      /** If set, calls this endpoint's virtual close() method on destruction */
      void set_close_on_destruction(bool value);

      /** True if this endpoint's virtual close() method will be called on destruction */
      bool close_on_destruction() const;

      /**
       * Close the I/O point and reopen to a new (or same state).
       *
       * If state is UNCHANGED will reopen to the previous state.
       *
       * close and reopen is accomplished by calling change_state(CLOSED) and then
       * change_state(new_state), not by calling close directly.
       */
      void close_and_reopen(long state = ENDPOINT_UNCHANGED);

      /** Signal emitted when this endpoint is opened */
      sigc::signal<void> signal_opened();

      /** Signal emitted when this endpoint is closed */
      sigc::signal<void> signal_closed();

      /** True if this endpoint is in an open state */
      bool is_open();

      /** True if this endpoint is in a closed state */
      bool is_closed();

    protected:
      Glib::Thread *m_read_thread;
      Glib::Thread *m_write_thread;
      Glib::Thread *m_read_delivery_thread;
      bool m_read_terminate;
      bool m_read_terminate_immediate;
      bool m_write_terminate;
      bool m_write_terminate_immediate;

      long int m_read_timeout;
      long int m_write_timeout;
      bool m_timestamp;

      Glib::Mutex m_read_queue_lock;
      Glib::Mutex m_write_queue_lock;

      Glib::Cond m_read_delivery_conditional;
      Glib::Cond m_write_conditional;

      Glib::Dispatcher* m_dispatcher;
      sigc::connection m_dispatcher_connection;

      bool m_use_dispatcher;

      sigc::signal<void, const Data> m_signal_data;
      sigc::signal<void, size_t> m_signal_data_received;
      sigc::signal<void, size_t> m_signal_data_transmitted;
      std::queue<Data> m_read_queue;
      std::queue<Data> m_write_queue;

      virtual void read_thread_main();
      virtual void read_thread_cleanup();

      virtual void read_delivery_thread_main();
      virtual void read_delivery_thread_cleanup();

      virtual void write_thread_main();
      virtual void write_thread_cleanup();

      void queue_received_data( const Data d );
      void queue_write_data( const Data d );
      void emit_received_data( );


      bool m_close_on_destruction;
      long m_state;
      bool m_readable;
      bool m_writable;

      virtual void set_state_opened();
      virtual void set_state_closed();

      sigc::signal<void> m_signal_opened;
      sigc::signal<void> m_signal_closed;

      virtual ssize_t write_data(long int timeout, const Data data) throw (write_exception) = 0;

      virtual Data read_data(long int timeout, size_t s = 0) throw (read_exception) = 0;

    };

}

Conexus::Endpoint& operator<<(Conexus::Endpoint& io, const Conexus::Data d);
Conexus::Endpoint::pointer operator<<(Conexus::Endpoint::pointer io, const Conexus::Data d);
Conexus::Endpoint& operator>>(Conexus::Endpoint& io, Conexus::Data d);
Conexus::Endpoint::pointer operator>>(Conexus::Endpoint::pointer io, Conexus::Data d);

#endif
