/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#ifndef CONEXUSENUMS_H
#define CONEXUSENUMS_H

#include <netinet/ip.h>
#include <termios.h>

namespace Conexus {

  /**
   * These enumerations are used in the socket class methods, and use is also encouraged in children.
   */
  typedef enum EndpointState {
    ENDPOINT_NOSTATE   = 0,
    ENDPOINT_CLOSED    = 1<<0,    /**< The Endpoint object is in a closed state */
    ENDPOINT_OPENED    = 1<<1,   /**< The Endpoint object is in an opened state */
    ENDPOINT_LAST_STATE  = ENDPOINT_OPENED,
    ENDPOINT_UNCHANGED = ~0, /**< setting this mode will keep the mode unchanged */
  } EndpointState;

  /**
   * These enumerations are used in the socket class methods, and use is also encouraged in children.
   */
  typedef enum SocketState {
    SOCKET_BOUND     = ENDPOINT_LAST_STATE << 1,  /**< The socket is bound to an interface */
    SOCKET_CONNECTED = ENDPOINT_LAST_STATE << 2,  /**< The socket is connected (TCP) */
    SOCKET_LISTENING = ENDPOINT_LAST_STATE << 3,  /**< The socket is listening for connections (TCP) */
    SOCKET_ACCEPTED  = ENDPOINT_LAST_STATE << 4,  /**< The socket is an accepted connection (TCP) */
    SOCKET_LAST_STATE  = SOCKET_ACCEPTED,
  } SocketState;

  typedef enum SerialMode {
    SERIAL_READ=1<<0,
    SERIAL_WRITE=1<<1,
    SERIAL_UNCHANGED=1<<2,
  } SerialMode;

  typedef enum PipeMode {
    PIPE_READ=ENDPOINT_LAST_STATE<<1,           /**< opened for reading */
    PIPE_WRITE=ENDPOINT_LAST_STATE<<2,            /**< opened for writing */
  } PipeMode;

  typedef enum FileMode {
    FILE_READ=1<<0,
    FILE_WRITE=1<<1,
    FILE_APPEND=1<<2,
    FILE_CREATE=1<<3,
    FILE_DIRECT=1<<4,
    FILE_LARGE_FILE=1<<5,
    FILE_NO_ACCESS_TIME=1<<6,
    FILE_NO_FOLLOW=1<<7,
    FILE_TRUNCATE=1<<8,
    FILE_EXCLUSIVE_CREATE=1<<9,
    FILE_UNCHANGED=1<<10,
  } FileMode;

  typedef enum QueueMode {
    QUEUE_READ=1<<0,
    QUEUE_WRITE=1<<1,
    QUEUE_CREATE=1<<2,
    QUEUE_EXCLUSIVE_CREATE=1<<3,
    QUEUE_NONBLOCKING=1<<4,
    QUEUE_UNCHANGED=1<<5,
  } QueueMode;

  typedef enum TOS {
    TOS_LOWDELAY = IPTOS_LOWDELAY,       /**< minimize delays for interactive traffic */
    TOS_THROUGHPUT = IPTOS_THROUGHPUT,   /**< optimize throughput */
    TOS_RELIABILITY = IPTOS_RELIABILITY, /**< optimize for reliability */
    TOS_LOWCOST = IPTOS_LOWCOST,         /**< for "filler data" where slow transmission doesn’t matter */
  } TOS;

  /**
   * Aliases for the tty function options relating to when an option change takes effect.
   */
  typedef enum SetOption {
    /** the setting change occurs immediately */
    SET_NOW = TCSANOW,

    /**
     * the setting change occurs after all output written to the object
     * has been transmitted, and all input that has been received but not
     * read will be discarded before the change is made.
     */
    SET_FLUSH = TCSAFLUSH,

    /**
     * the setting change occurs after all output has been transmitted;
     * This function should be used when changing parameters that affect output.
     */
    SET_DRAIN = TCSADRAIN
  } SetOption;

  typedef enum Parity {
    PARITY_NONE,  /** No parity check */
    PARITY_ODD,   /** Odd parity checks */
    PARITY_EVEN,  /** Even parity checks */
  } Parity;

  typedef enum ParityError {
    PARITY_ERROR_DISCARD,  /** Discards all parity error bytes */
    PARITY_ERROR_MARK,     /** Delivers three bytes (0xFF, 0x00 and c; where c is the error byte */
    PARITY_ERROR_ZERO,     /** Delivers 0x00 when a parity error is detected */
    PARITY_ERROR_IGNORE,   /** Ignores all parity errors and delivers all data as is (this is not the same as IGNPAR) */
  } ParityError;

  typedef enum FlowControl {
    FLOW_NONE,  /** CRTSCTS off, IXON off, IXOFF off */
    FLOW_HARDWARE,  /** CRTSCTS on, IXON off, IXOFF off */
    FLOW_SOFTWARE,  /** CRTSCTS off, IXON on, IXOFF on */
    FLOW_CUSTOM,  /** Combinations of CRTSCTS, IXON, IXOFF other than FLOW_NONE, FLOW_HARDWARE, or FLOW_SOFTWARE */
  } FlowControl;


}

#endif
