/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef CONEXUSEXCEPT_CONNECT_H
#define CONEXUSEXCEPT_CONNECT_H

#include <conexus/except.h>

namespace Conexus {

/**
 * Base class for all connect exceptions.
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class connect_exception: public conexus_exception {
public:
connect_exception(int e=0): conexus_exception("Unknown exception, socket not connected.",e,"connect") { }
connect_exception(const std::string s, int e=0): conexus_exception(s,e,"connect") { }
}
;

namespace exception {


namespace connect {


/**
 * EBADF
 * Thrown when equivalent UNIX domain (AF_UNIX) API error is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class bad_fd: public connect_exception {
public:
  bad_fd(): connect_exception("The file descriptor is not a valid index in the descriptor table.", EBADF) { }
}
;

/**
 * EFAULT
 * Thrown when equivalent UNIX domain (AF_UNIX) API error is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class segfault: public connect_exception {
public:
  segfault(): connect_exception("The socket structure  address  is  outside  the  user’s  address space.", EFAULT) { }
}
;

/**
 * ENOTSOCK
 * Thrown when equivalent UNIX domain (AF_UNIX) API error is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class not_socket: public connect_exception {
public:
  not_socket(): connect_exception("The file descriptor is not associated with a socket.", ENOTSOCK) { }
}
;

/**
 * EISCONN
 * Thrown when equivalent UNIX domain (AF_UNIX) API error is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class connected_socket: public connect_exception {
public:
  connected_socket(): connect_exception("The socket is already connected.", EISCONN) { }
}
;

/**
 * ECONNREFUSED
 * Thrown when equivalent UNIX domain (AF_UNIX) API error is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class connection_refused: public connect_exception {
public:
  connection_refused(): connect_exception("No one listening on the remote address.", ECONNREFUSED) { }
}
;

/**
 * ETIMEDOUT
 * Thrown when equivalent UNIX domain (AF_UNIX) API error is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class timeout: public connect_exception {
public:
  timeout(): connect_exception("Timeout while attempting connection.", ETIMEDOUT) { }
}
;

/**
 * ENETUNREACH
 * Thrown when equivalent UNIX domain (AF_UNIX) API error is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class network_unreachable: public connect_exception {
public:
  network_unreachable(): connect_exception("Network is unreachable.", ENETUNREACH) { }
}
;

/**
 * EADDRINUSE
 * Thrown when equivalent UNIX domain (AF_UNIX) API error is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class address_in_use: public connect_exception {
public:
  address_in_use(): connect_exception("Local address is already in use.", EADDRINUSE) { }
}
;

/**
 * EINPROGRESS
 * Thrown when equivalent UNIX domain (AF_UNIX) API error is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class not_immediate: public connect_exception {
public:
  not_immediate(): connect_exception("The socket is non-blocking and the connection cannot be completed immediately.", EINPROGRESS) { }
}
;

/**
 * EALREADY
 * Thrown when equivalent UNIX domain (AF_UNIX) API error is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class in_progress: public connect_exception {
public:
  in_progress(): connect_exception("The socket is non-blocking and a previous connection attempt has not yet been completed.", EALREADY) { }
}
;

/**
 * EAGAIN
 * Thrown when equivalent UNIX domain (AF_UNIX) API error is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class try_again: public connect_exception {
public:
  try_again(): connect_exception("No more free local ports or insufficient entries in the routing cache.", EAGAIN) { }
}
;

/**
 * EAFNOSUPPORT
 * Thrown when equivalent UNIX domain (AF_UNIX) API error is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class af_not_supported: public connect_exception {
public:
  af_not_supported(): connect_exception("The passed address didn’t have the correct address family in its sa_family field.", EAFNOSUPPORT) { }
}
;

/**
 * EACCES or EPERM
 * Thrown when equivalent UNIX domain (AF_UNIX) API error is encountered. May be caught alone
 * or via parent class.
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class access: public connect_exception {
public:
  access(): connect_exception("The user tried to connect to a broadcast address without having the socket broadcast flag enabled or the connection request failed because of a local firewall rule.", EACCES) { }
}
;

/**
 * Thrown when attempting to autoconnect to a socket that is not capable of
 * autoconnecting or does not have autoconnection enabled.
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class no_remote_address: public connect_exception {
public:
  no_remote_address(): connect_exception("Socket cannot auto connect because a default remote address has not been set.", -1) { }
};

/**
 * Thrown when connect is requested of the Socket class
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class no_socket_connect: public connect_exception
{
public:
  no_socket_connect(): connect_exception("The Socket class connect() method cannot be called without an address.", -1)
  { }
};

/**
 * Thrown when connect is requested and a socket cannot be opened
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class not_open: public connect_exception
{
public:
  not_open(): connect_exception("Socket could not be opened for connect() method.", -1)
  { }
};

/**
 * Thrown when connect is requested of the UDPPoset class
 * @author Rick L Vinyard Jr
 * @ingroup exceptions
 */
class no_udpposet_connect: public connect_exception
{
  public:
    no_udpposet_connect(): connect_exception("The UDPPoset class connect() method cannot be called.", -1)
    { }
};



}
}
}

#endif // CONEXUSCONNECT_ERROR_H
