/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef CONEXUSEXCEPT_READ_H
#define CONEXUSEXCEPT_READ_H

#include <conexus/except.h>

namespace Conexus
  {

  /**
   * Base class for all receiving exceptions.
   * @author Rick L Vinyard Jr
   * @ingroup exceptions
   */
  class read_exception: public conexus_exception
    {
    public:
    read_exception(int e=0): conexus_exception("Unknown exception, read failed.",e,"read")
      { }
    read_exception(const std::string s, int e=0): conexus_exception(s,e,"read")
      { }
    }
  ;

  namespace exception
    {

    namespace read
      {

      /**
       * EBADF: Thrown when equivalent BSD socket API error is encountered. May be caught alone
       * or via parent class.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class bad_fd: public read_exception
        {
        public:
          bad_fd(): read_exception("sockfd is not a valid descriptor.", EBADF)
          { }
        }
      ;

      /**
       * ECONNREFUSED
       * A remote host refused to allow the network connection (typically
       * because it is not running the requested service).
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class connection_refused: public read_exception
        {
        public:
          connection_refused(): read_exception("A remote host refused to allow the network connection.", ECONNREFUSED)
          { }
        }
      ;

      /**
       * ENOTCONN
       * The socket is associated with a connection-oriented protocol and
       * has not been connected (see connect(2) and accept(2)).
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class not_connected: public read_exception
        {
        public:
          not_connected(): read_exception("The socket is associated with a connection-oriented protocol and has not been connected", ENOTCONN)
          { }
        }
      ;

      /**
       * ENOTSOCK
       * The argument s is not a socket.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class not_socket: public read_exception
        {
        public:
          not_socket(): read_exception("The argument s is not a socket.", ENOTSOCK)
          { }
        }
      ;

      /**
       * EAGAIN
       * The  socket  is  marked  non-blocking  and the receive operation
       * would block, or a receive timeout had been set and  the  timeout
       * expired before data was received.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class would_block: public read_exception
        {
        public:
          would_block(): read_exception("The socket is marked non-blocking and  the  requested  operation would block.", EAGAIN)
          { }
        }
      ;

      /**
       * EINTR
       * The  receive  was interrupted by delivery of a signal before any
       * data were available.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class interrupted: public read_exception
        {
        public:
          interrupted(): read_exception("The  receive  was interrupted by delivery of a signal before any data were available.", EINTR)
          { }
        }
      ;

      /**
       * EFAULT
       * An invalid user space address was specified for a parameter.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class invalid_user_space_address: public read_exception
        {
        public:
          invalid_user_space_address(): read_exception("An invalid user space address was specified for a parameter.", EFAULT)
          { }
        }
      ;

      /**
       * EINVAL
       * Invalid argument passed.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class invalid_argument: public read_exception
        {
        public:
          invalid_argument(): read_exception("Invalid argument passed.", EINVAL)
          { }
        }
      ;

      /**
       * ENOMEM
       * No memory available.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class no_memory: public read_exception
        {
        public:
          no_memory(): read_exception("No memory available.", ENOMEM)
          { }
        }
      ;

      /**
       * Not opened.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class not_opened: public read_exception
      {
      public:
      not_opened(): read_exception("Trying to read from a device that can't be opened.", -1)
        { }
      }
      ;

      /**
       * Not bound.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class not_bound: public read_exception
      {
      public:
      not_bound(): read_exception("Trying to read from a socket that can't be bound.", -1)
        { }
      }
      ;

      /**
       * Disconnected.
       *
       * Trying to read on a socket where the peer has performed an orderly shutdown.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class disconnected: public read_exception
      {
        public:
          disconnected(): read_exception("Trying to read from a socket where the peer has performed an orderly shutdown.", -1)
          { }
      }
      ;

      /**
       * Timeout.
       *
       * The read operation timed out.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class timeout: public read_exception
      {
        public:
          timeout(): read_exception("The read operation timed out.", -1)
          { }
      }
      ;

      /**
       * End of file.
       *
       * An end of file condition occurred.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class eof: public read_exception
      {
      public:
      eof(): read_exception("End of file.", -1)
        { }
      }
      ;


    }

  }

}

#endif // CONEXUSSEND_ERROR_H
