/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef CONEXUSEXCEPT_WRITE_H
#define CONEXUSEXCEPT_WRITE_H

#include <conexus/except.h>

namespace Conexus
  {

  /**
   * Base class for all writing errors.
   * @author Rick L Vinyard Jr
   * @ingroup exceptions
   */
  class write_exception: public conexus_exception
    {
    public:
      write_exception(int e=0): conexus_exception("Unknown exception, write failed.",e,"write")
      { }
      write_exception(const std::string s, int e=0): conexus_exception(s,e,"write")
      { }
    }
  ;

  namespace exception
    {

    namespace write
      {

      /**
       * EAGAIN or EWOULDBLOCK
       * The socket is marked non-blocking and  the  requested  operation
       * would block.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class would_block: public write_exception
        {
        public:
          would_block(): write_exception("The socket is marked non-blocking and  the  requested  operation would block.", EWOULDBLOCK)
          { }
        }
      ;

      /**
       * EBADF: Thrown when equivalent BSD socket API error is encountered. May be caught alone
       * or via parent class.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class bad_fd: public write_exception
        {
        public:
          bad_fd(): write_exception("sockfd is not a valid descriptor.", EBADF)
          { }
        }
      ;

      /**
       * ECONNRESET
       * Connection reset by peer.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class connection_reset: public write_exception
        {
        public:
          connection_reset(): write_exception("Connection reset by peer.", ECONNRESET)
          { }
        }
      ;

      /**
       * EDESTADDRREQ
       * The socket is not connection-mode, and no peer address is set.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class destination_address_required: public write_exception
        {
        public:
          destination_address_required(): write_exception("The socket is not connection-mode, and no peer address is set.", EDESTADDRREQ)
          { }
        }
      ;


      /**
       * EFAULT
       * An invalid user space address was specified for a parameter.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class invalid_user_space_address: public write_exception
        {
        public:
          invalid_user_space_address(): write_exception("An invalid user space address was specified for a parameter.", EFAULT)
          { }
        }
      ;

      /**
       * EINTR
       * A signal occurred before any data was transmitted.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class interrupted: public write_exception
        {
        public:
          interrupted(): write_exception("A signal occurred before any data was transmitted.", EINTR)
          { }
        }
      ;


      /**
       * EINVAL
       * Invalid argument passed.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class invalid_argument: public write_exception
        {
        public:
          invalid_argument(): write_exception("Invalid argument passed.", EINVAL)
          { }
        }
      ;

      /**
       * EISCONN
       * The connection-mode socket was connected already but a recipient
       * was specified.  (Now either  this  error  is  returned,  or  the
       * recipient specification is ignored.)
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class is_connected: public write_exception
        {
        public:
          is_connected(): write_exception("The connection-mode socket was connected already but a recipient was specified.", EISCONN)
          { }
        }
      ;


      /**
       * EMSGSIZE
       * The  socket  type  requires that message be sent atomically, and
       * the size of the message to be sent made this impossible.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class message_size: public write_exception
        {
        public:
          message_size(): write_exception("The  socket  type  requires that message be sent atomically, and the size of the message to be sent made this impossible.", EMSGSIZE)
          { }
        }
      ;


      /**
       * ENOBUFS
       * The output queue for a network interface was full.  This  gener-
       * ally  indicates  that the interface has stopped writing, but may
       * be caused by transient congestion.   (Normally,  this  does  not
       * occur  in Linux. Packets are just silently dropped when a device
       * queue overflows.)
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class no_buffers: public write_exception
        {
        public:
          no_buffers(): write_exception("The output queue for a network interface was full.", ENOBUFS)
          { }
        }
      ;


      /**
       * ENOMEM
       * No memory available.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class no_memory: public write_exception
        {
        public:
          no_memory(): write_exception("No memory available.", ENOMEM)
          { }
        }
      ;

      /**
       * EACCES
       * Permission denied.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class permission_denied: public write_exception
        {
        public:
          permission_denied(): write_exception("Permission denied.", EACCES)
          { }
        }
      ;

      /**
       * ENOTCONN
       * The socket is not connected, and no target has been given.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class not_connected: public write_exception
        {
        public:
          not_connected(): write_exception("The socket is not connected, and no target has been given.", ENOTCONN)
          { }
        }
      ;

      /**
       * ENOTSOCK
       * The argument s is not a socket.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class not_socket: public write_exception
        {
        public:
          not_socket(): write_exception("The argument s is not a socket.", ENOTSOCK)
          { }
        }
      ;


      /**
       * EOPNOTSUPP
       * Some bit in the flags argument is inappropriate for  the  socket
       * type.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class operation_not_supported: public write_exception
        {
        public:
          operation_not_supported(): write_exception("Some bit in the flags argument is inappropriate for  the  socket type.", EOPNOTSUPP)
          { }
        }
      ;


      /**
       * EPIPE
       * The  local  end  has  been  shut  down  on a connection oriented
       * socket.  In this case the process will also  receive  a  SIGPIPE
       * unless MSG_NOSIGNAL is set.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class pipe: public write_exception
        {
        public:
          pipe(): write_exception("The  local  end  has  been  shut  down  on a connection oriented socket.", EPIPE)
          { }
        }
      ;

      /**
       * ECONNREFUSED
       * With writes, this is most likely triggered by an ICMP UNREACHABLE message.
       * Therefore, the first one (or even several) messages may transmit before
       * the ICMP mechanism reports UNREACHABLE.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class connection_refused: public write_exception
        {
        public:
          connection_refused(): write_exception("The connection has been refused. This is probably the result of an ICMP UNREACHABLE message, so the first few transmissions may have actually gone out before the ICMP UNREACHABLE message was processed. This probably means that the destination (host or port) doesn't exist or isn't active.", ECONNREFUSED)
          { }
        }
      ;

      /**
       * Thrown when attempting to send without specifying a default address.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class no_default_remote_address: public write_exception
        {
        public:
          no_default_remote_address(): write_exception("Socket cannot send because a default remote address has not been set.", -1)
          { }
        }
      ;

      /**
       * Thrown when sendto without an address is requested of the Socket class
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class no_sendto_without_address: public write_exception
        {
        public:
          no_sendto_without_address(): write_exception("Socket is not capable of sendto without an address.", -1)
          { }
        }
      ;

      /**
       * Writing on an I/O than can't open
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class not_opened: public write_exception
        {
        public:
          not_opened(): write_exception("Trying to write on I/O that cannot open.", -1)
          { }
        }
      ;

      /**
       * Writing to a read-only I/O
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
       */
      class read_only: public write_exception
        {
        public:
          read_only(): write_exception("Trying to write to a read only I/O.", -1)
          { }
        }
      ;

            /**
       * Timeout.
       *
       * The write operation timed out.
       * @author Rick L Vinyard Jr
       * @ingroup exceptions
             */
      class timeout: public write_exception
      {
        public:
          timeout(): write_exception("The write operation timed out.", -1)
          { }
      }
      ;



    }

  }

}

#endif // CONEXUSSEND_ERROR_H
