/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#ifndef CONEXUSFILE_H
#define CONEXUSFILE_H

#include <conexus/enums.h>
#include <conexus/filedescriptor.h>

/**
 * \page manual_file File Endpoints
 */

namespace Conexus
{

  /**
   * This class encapsulates a file.
   *
   * @author Rick L Vinyard Jr
   * @ingroup conexus
   */
  class File : public FileDescriptor
  {
    protected:

      File( const std::string& filename,
            long unsigned mode );

    public:
      typedef ConexusPointer<File> pointer;

      static File::pointer create( const std::string& filename = std::string(),
                                   long unsigned mode = FILE_READ | FILE_WRITE | FILE_CREATE | FILE_TRUNCATE );

      virtual ~File();

      /**
       * Opens the named file in the specified mode.
       */
      virtual void open( const std::string name,
                         long unsigned mode = FILE_UNCHANGED )
          throw ( open_exception);

      /**
       * Will attempt to open the file if a filename has been specified. Otherwise an
       * exception will be thrown.
       */
      virtual void open() throw ( open_exception );

      /** Will return the filename. */
      std::string filename();

      long unsigned mode();

      bool eof();

      bool throw_eof();

      void set_throw_eof( bool teof = true );

      /**
       * Sets the filename.
       *
       * If the file is currently open and d is not equal to the current filename,
       * the current file will be closed.
       *
       * Also, if auto_open is set, the new file will be immediately opened.
       *
       * Additionally, this method checks whether the parameter d is the same as the current filename,
       * and parameter mode is the same as the current mode.
       * If both are equal, the file will not be closed and reopened unless force=true.
       */
      void set_filename( std::string d, long unsigned mode = FILE_UNCHANGED ) throw ( open_exception );

      sigc::signal<void> signal_filename_changed();

    protected:
      std::string m_filename;
      long unsigned m_mode;
      bool m_eof;
      bool m_throw_eof;

      sigc::signal<void> m_signal_filename_changed;

      virtual Data read_data( long int timeout, size_t s = 0 ) throw ( read_exception );

  };

}

#endif
