/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#ifndef CONEXUS_IPV4TCP_H
#define CONEXUS_IPV4TCP_H

#include <conexus/ipv4_ip.h>

/**
 * \page manual_ipv4_tcp IPv4 TCP Endpoint
 *
 * \section example_ipv4_tcp_client IPv4 TCP Client Example
 * \include ipv4_tcp_client.cpp
 *
 * \section example_ipv4_tcp_server IPv4 TCP Server Example
 * \include ipv4_tcp_server.cpp
 *
 * \section example_ipv4_tcp_echo_client IPv4 TCP Echo Client Example
 * \include ipv4_tcp_echo_client.cpp
 *
 * \section example_ipv4_tcp_echo_server IPv4 TCP Echo Server Example
 * \include ipv4_tcp_echo_server.cpp
 */

namespace Conexus
{

  namespace IPv4
  {

    /**
     * This class encapsulates a socket used for TCP/IP communications.
     *
     * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
     * @ingroup IPv4
     */
    class TCP : public IP
    {

      protected:

        friend class TCPServer;

        TCP( uint32_t address, uint16_t port );

        TCP( const std::string& address, uint16_t port );

      public:

        typedef ConexusPointer<TCP> pointer;

        static pointer create( uint16_t port = 0 );

        static pointer create( uint32_t address, uint16_t port );

        static pointer create( const std::string& address, uint16_t port = 0 );

        virtual ~TCP() throw();

        virtual ssize_t input_available();

        /**
         * Returns the size in bytes of the local send queue.
         */
        int pending_output_size();

      protected:

        /**
         * Protected constructor to be used by TCPServer to create TCP connections.
         * The second bool parameter is used to differentiate the public method from the private method.
         */
        TCP( int accepted_fd, bool );

        static TCP::pointer create( int accepted_fd, bool );

        virtual void on_local_address_changed( );
        virtual void on_remote_address_changed( );

        /**
         * Overloads the Socket ancestor send method to ensure that the socket is connected
         * before trying to send.
         */
        virtual ssize_t write_data( long int timeout, const Data data ) throw ( write_exception );

        /**
         * Receives a single datagram from the TCP socket up to s bytes in size; allocates
         * any memory necessary as a smart pointer contained in the returned Data object.
         *
         * If s = 0 then receives the next pending datagram, however large it may be.
         */
        virtual Data read_data( long int timeout, size_t s = 0 ) throw ( read_exception );

    };

  }

}

#endif
