/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef CONEXUS_IPV6IP_H
#define CONEXUS_IPV6IP_H

#include <conexus/ip.h>
#include <conexus/ipv6_address.h>

/**
 * \page manual_ipv6 IPv6 Endpoint
 */

namespace Conexus
{

  namespace IPv6
  {
    /**
     * @defgroup IPv6 Classes for communicating via IP version 6 sockets
     */


    /**
     * This class serves as the base class for all IPv4 communication classes.
     *
     * All child classes such as UDP and TCP inherit the association of a local
     * host address from this class as well as the concept of address association,
     * multicasting capabilities and of course, ports.
     *
     * This class also stores an internal address object representing a local address
     * object. All BSD socket API functions (such as bind) that require a sockaddr_in
     * structure will be provided this internal attribute. Therefore, it is the
     * responsibility of children such as UDP and TCP to make the appropriate modifications
     * to this protected attribute.
     *
     * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
     * @ingroup IPv6
     */
    class IP : public Conexus::IP
    {

      protected:

        /**
         * Default constructor which sets the socket domain to PF_INET6 but does not provide
         * a socket type or protocol.
         */
        IP( int type = -1, int protocol = 0 );

      public:
        /** Class typedef to smart pointer */
        typedef ConexusPointer<IP> pointer;

        virtual ~IP() throw();

        /**
         * Attempts to bind this socket to the internal address
         */
        virtual void bind() throw ( bind_exception );

        virtual void bind( const Conexus::Address& a ) throw ( bind_exception );

        virtual void connect() throw ( connect_exception );

        virtual void connect( const Address& a ) throw ( connect_exception );

        virtual Address& local_address();

        virtual Address& remote_address();

        virtual IPAddress& generic_local_address();

        virtual IPAddress& generic_remote_address();

        virtual void set_remote_address( const Address& addr );

        virtual void set_remote_address( uint32_t addr, uint16_t port );

        virtual void set_remote_address( const struct in6_addr& addr, uint16_t port );

        virtual void set_remote_address( const std::string& addr, uint16_t port );

        virtual void unset_remote_address();

        virtual void set_local_address( const Address& addr );

        virtual void set_local_address( uint32_t addr, uint16_t port );

        virtual void set_local_address( const struct in6_addr& addr, uint16_t port );

        virtual void set_local_address( const std::string& addr, uint16_t port );

      protected:
        Address m_local_address;
        Address m_remote_address;
        bool m_remote_address_set;

        virtual void on_local_address_changed( );
        virtual void on_remote_address_changed( );

        void on_local_address_changed_proxy( );
        void on_remote_address_changed_proxy( );
    };

  }

}

#endif
