/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef CONEXUS_IPV6UDP_H
#define CONEXUS_IPV6UDP_H

#include <set>

#include <conexus/ipv6_ip.h>

/**
 * \page manual_ipv6_udp IPv6 UDP Endpoint
 *
 * \section example_ipv6_udp_client IPv6 UDP client example
 * \include ipv6_udp_client.cpp
 *
 * \section example_ipv6_udp_server IPv6 UDP server example
 * \include ipv6_udp_server.cpp
 *
 * \section example_ipv6_udp_multicast_client IPv6 UDP multicast client example
 * \include ipv6_multicast_client.cpp
 *
 * \section example_ipv6_udp_multicast_server IPv6 UDP multicast server example
 * \include ipv6_multicast_server.cpp
 */

namespace Conexus
{

  namespace IPv6
  {

    /**
     * This class encapsulates a socket used for UDP/IP communications.
     *
     * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
     * @ingroup IPv6
     */
    class UDP : public IP
    {
      protected:

        UDP( const struct in6_addr& local_if, uint16_t localport, uint16_t remoteport );

        UDP( const std::string& local_if, uint16_t localport, uint16_t remoteport );

      public:

        typedef ConexusPointer<UDP> pointer;

        static pointer create( uint16_t localport = 0, uint16_t remoteport = 0 );

        static pointer create( const struct in6_addr& local_if, uint16_t localport = 0, uint16_t remoteport = 0 );

        static pointer create( const std::string& local_if, uint16_t localport = 0, uint16_t remoteport = 0 );

        virtual ~UDP() throw();

        virtual ssize_t input_available();

        virtual void bind() throw ( bind_exception );

        virtual void bind( const Conexus::Address& a ) throw ( bind_exception );

        virtual void close() throw ( close_exception );

        /**
         * Returns the size in bytes of the local send queue.
         */
        virtual int output_pending();

        virtual void connect() throw ( connect_exception );

        virtual void connect( const Address& a ) throw ( connect_exception );

        /**
         * There are some circumstances in which it may be desirable to set a
         * default address and have the send method transparently call the sendto method
         * without connecting rather than connecting and calling the send method.
         *
         * A specific example of this is when the remote address/port may occassionally
         * be unavailable. In this case the destination may generate an ICMP UNREACHABLE
         * message, and in accordance with RFC 1122 the local machine will fail any
         * future sends on a connected port, throwing a write::connection_refused exception.
         * If you want to ensure that the datagram is sent contrary to RFC 1122 you could
         * set the default address, set send without connect and call send without throwing
         * connection exceptions.
         *
         * But, since this is contrary to RFC 1122 behavior you should really think about
         * your design before setting this.
         */

        virtual void set_write_without_connect( bool value = true );

        virtual bool write_without_connect() const;

        void add_multicast_interface( unsigned iface_index );

        void add_multicast_interface( const std::string& iface_name );

        void remove_multicast_interface( unsigned iface_index );

        void remove_multicast_interface( const std::string& iface_name );

        bool is_multicast_interface( unsigned iface_index );

        bool is_multicast_interface( const std::string& iface_name );

        virtual bool suppress_multicast_join() const;

        virtual void set_suppress_multicast_join( bool value = true );

        /**
         * Returns the multicast ttl of the currently associated address.
         */
        virtual int multicast_hops();

        /**
         * Sets the multicast ttl of the currently associated address to @em hops.
         */
        virtual void set_multicast_hops( int hops );

        static unsigned int default_multicast_hops();

        static void set_default_multicast_hops(unsigned int hops);

        /**
         * If true multicast packets are looped back to local sockets.
         */
        virtual bool multicast_loopback();

        /**
         * If parameter @em loop is true then multicast packets are looped back to local sockets.
         */
        virtual void set_multicast_loopback( bool loop );

        static bool default_multicast_loopback();

        static void set_default_multicast_loopback(bool loopback);

        /**
         * All data output on this socket is accumulated into a single datagram that is transmitted
         * when the socket is uncorked.
         */
        //       void cork();

        /**
         * All data output on this socket is accumulated into a single datagram that is transmitted
         * when the socket is uncorked.
         */
        //       void uncork();

        //       bool is_corked();

      protected:

        static unsigned int m_default_hops;
        static bool m_default_loopback;

        bool m_write_without_connect;
        bool m_suppress_multicast_join;
        std::set<unsigned> m_multicast_interfaces;

        virtual void on_local_address_changed( );
        virtual void on_remote_address_changed( );

        /**
         * Issues a join on all multicast interfaces.
         * If no interfaces have been specified, then the join is issued on the
         * system default.
         */
        virtual void join();

        /**
         * Issues a join on the local address for the specified interface.
         */
        virtual void join(unsigned interface);

        /**
         * Issues a leave on all multicast interfaces.
         * If no interfaces have been specified, then the leave is issued on the
         * system default.
         */
        virtual void leave();

        /**
         * Issues a leave on the local address for the specified interface.
         */
        virtual void leave(unsigned interface);

        /**
         * Overloads the Socket ancestor send method to ensure that the socket is connected
         * before trying to send.
         */
        virtual ssize_t write_data( long int timeout, const Data data ) throw ( write_exception );

        /**
         * Receives a single datagram from the UDP socket up to s bytes in size; allocates
         * any memory necessary as a smart pointer contained in the returned Data object.
         *
         * If s = 0 then receives the next pending datagram, however large it may be.
         */
        // virtual Data::pointer read_data(long int timeout, size_t s = 0) throw (read_exception);
    };

  }

}

#endif
