/***************************************************************************
 *   Copyright (C) 2007 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef CONEXUSMESSAGEQUEUE_H
#define CONEXUSMESSAGEQUEUE_H

#include <mqueue.h>
#include <sys/stat.h>

#include <conexus/endpoint.h>

/**
 * \page manual_messagequeue Message Queue Endpoints
 *
 * \section example_messagequeue_client Message Queue Client Example
 * \include messagequeue_client.cpp
 *
 * \section example_messagequeue_server Message Queue Server Example
 * \include messagequeue_server.cpp
 */

namespace Conexus
{

  /**
   * This class encapsulates a POSIX message queue.
   *
   * \todo Drive message queues off notifications rather than the timeout mechanism of Endpoint
   *
   * @author Rick L Vinyard Jr
   * @ingroup conexus
   */
  class MessageQueue : public Endpoint
  {
    protected:

      MessageQueue( const std::string& name,
                    unsigned flags,
                    mode_t mode,
                    bool close_on_destruction
                  );

    public:

      typedef ConexusPointer<MessageQueue> pointer;

      static MessageQueue::pointer create( const std::string& name,
                                           unsigned flags=QUEUE_READ|QUEUE_WRITE|QUEUE_CREATE,
                                           mode_t mode = S_IRWXU,
                                           bool close_on_destruction=true
                                         );

      virtual ~MessageQueue();

      virtual void open() throw ( open_exception );

      virtual void close( bool force = false ) throw ( close_exception );

      const std::string& name() const;

      void set_name( const std::string& name );

      unsigned flags() const;

      void set_flags(unsigned flags);

      bool unlink_on_destruction();

      void set_unlink_on_destruction( bool u=true );

      long int max_messages();

      long int max_message_size();

    protected:
      std::string m_name;
      unsigned m_flags;
      bool m_unlink_on_destruction;
      long int m_max_messages;
      long int m_max_message_size;

      mode_t m_mode;

      mqd_t m_queue;

      virtual ssize_t write_data( long int timeout, const Data data ) throw ( write_exception );

      virtual Data read_data( long int timeout, size_t s = 0 ) throw ( read_exception );
  };

}

#endif
